# Warning: Do not save data to this file when this file name is 'config_sample.py' and it under your home directory.
#          Cause it can be rewritten by program automatic.


# verpathc_path = r'Z:\verpatch.exe'


# upx_dir = r'Z:\\'


# output_dir = r'Z:\wpi'


# output_filename = 'wpi'
